# saMDI.tcl version 1.0a3
# Copyright (C) 1998  Sam Tregar (sam@tregar.com)
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version. 
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# See README for usage details and contact information

package require stooop 3

# class mdi --
#
#   This class controls the main MDI window
#
# Arguments (can be abreviated uniquely):
#  Required:
#   -title - title to be displayed by the main MDI window
#   -height - height of the MDI window
#   -width - width of the MDI window
#
#  Optional:
#   -foreground and -background - the foreground and background
#       of the main window, inherited by slaves
#       default to white on blue
#   -clicktofocus - 1 is like Windows and Mac, 0 is like some Unix.
#       default is platform specific
#   -toplevel (optional) - an empty toplevel to use, if not specified mdi
#       creates its own.
#   -repositionIconsOnResize (optional) - Should I reshuffle the icons when 
#       the user resizes the mdi main window?  Defaults to 1.  If not icon
#       position can end up off the screen.
#
# Results:
#   Returns the object ID for the MDI window that is used in all future
#   calls to ::mdi functions and variables.
#   displays MDI window
class mdi {
    proc mdi {this args} {
        global tcl_platform

        if ![info exists mdi::(globalInitDone)] {
            mdi::globalInit
        }

        set mdi::($this,usetoplevel) ""
        set mdi::($this,title) ""
        set mdi::($this,height) ""
        set mdi::($this,width) ""
        set mdi::($this,foreground) white
        set mdi::($this,background) blue
        set mdi::($this,repositionIconsOnResize) 1

        if [info exists DEBUG-FORCE-PLATFORM] {
            set mdi::($this,platform) $DEBUG-FORCE-PLATFORM
        } else {
            set mdi::($this,platform) $tcl_platform(os)
        }

        switch -glob -- $mdi::($this,platform) {
            "Win*" {
                set mdi::($this,clicktofocus) 1
            }
            "Mac*" {
                set mdi::($this,clicktofocus) 1
            }
            default {
                # Unix and others
                set mdi::($this,clicktofocus) 0
            }
        }


        foreach {name value} $args {
            switch -glob -- $name {
                "-b*" {set mdi::($this,background) $value}
                "-click*" {set mdi::($this,clicktofocus) $value}
                "-f*" {set mdi::($this,foreground) $value}
                "-h*" {set mdi::($this,height) $value}
                "-r*" {set mdi::($this,repositionIconsOnResize) $value}
                "-ti*" {set mdi::($this,title) $value}
                "-to*" {set mdi::($this,usetoplevel) $value}
                "-w*" {set mdi::($this,width) $value}
                default {bgerror "::mdi class constructor : Unknown option ${name}!"}
            }
        }
        foreach name "title height width" {
            if ![info exists mdi::($this,$name)] {
                bgerror "::mdi class constructor : No value specified for required option ${name}!"
                return 0
            }
        }
        mdi::init $this
    }
    proc ~mdi {this} {
        foreach slave $mdi::($this,slaves) {
            delete $slave
        }
        destroy $mdi::($this,toplevel)
    }
}

# mdi::init --
#
#   Internal function to initialize an mdi instance.
#
# Arguments:
#   this
# Results:
#   mdi is initialized
#
proc mdi::init {this} {
    if ![string length $mdi::($this,usetoplevel)] {
        set mdi::($this,toplevel) [toplevel .mdi$this -background $mdi::($this,background) -height $mdi::($this,height) -width $mdi::($this,width)]
    } else {
        set mdi::($this,toplevel) $mdi::($this,usetoplevel)
        $mdi::($this,usetoplevel) configure -background $mdi::($this,background) -height $mdi::($this,height) -width $mdi::($this,width)
    }
    mdi::hide $this
    wm title $mdi::($this,toplevel) $mdi::($this,title)
    wm geometry $mdi::($this,toplevel) $mdi::($this,width)x$mdi::($this,height)+10+10
    set mdi::($this,core) [canvas $mdi::($this,toplevel).core -background $mdi::($this,background) -height $mdi::($this,height) -width $mdi::($this,width)]
    
    grid $mdi::($this,core) -row 0 -column 0 -sticky nswe
    grid rowconfigure $mdi::($this,toplevel) 0 -weight 1
    grid columnconfigure $mdi::($this,toplevel) 0 -weight 1

    bind $mdi::($this,toplevel) <Configure> "mdi::resizeHandler $this %h %w %W"
    mdi::show $this
}

# mdi::hide --
#
#   internal function - hides an MDI window
#
# Arguments:
#   this
# Results:
#   mdi window is unmapped from the display
#
proc mdi::hide {this} {
    wm withdraw $mdi::($this,toplevel)
}

# mdi::show --
#
#   Internal function - displays mdi window
#
# Arguments:
#   this
# Results:
#   mdi window is displayed on the screen
#
proc mdi::show {this} {
    wm deiconify $mdi::($this,toplevel)
    raise $mdi::($this,toplevel)
    update idletasks
}


# mdi::showSlave --
#
#   Internal function - displays mdiSlave window
#
# Arguments:
#   this
# Results:
#   mdiSlave window is displayed on the screen
#
proc mdi::showSlave {this slave} {
    set mdiSlave::($slave,window) [set mdi::($this,slaveWindow,$slave) [$mdi::($this,core) create window $mdiSlave::($slave,x) $mdiSlave::($slave,y) -window $mdiSlave::($slave,outerFrame) -anchor nw -tags [list slaveWindow mdiSlave$this]]]

    # Hm.  How to make this work...
    #if $mdiSlave::($slave,maximized) {
    #    mdiSlave::maximize $slave
    #}

    update
}

# mdi::resizeHandler --
#
#   Internal proc - Handles <Configure> events in the main window.
#
# Arguments:
#   this height width window
# Results:
#   Repositions icons (not done yet)
#
proc mdi::resizeHandler {this height width window} {
    if [string compare $mdi::($this,toplevel) $window] {
        return
    }
    if {($width == $mdi::($this,width)) && ($height == $mdi::($this,height))} {
        return
    }
    set mdi::($this,width) $width
    set mdi::($this,height) $height
    catch {unset mdi::($this,iconSpots)}
    if {$mdi::($this,repositionIconsOnResize)} {
        foreach slave $mdi::($this,slaves) {
            if {$mdiSlave::($slave,iconized)} {
                foreach {x y} [mdiSlave::findIconSpot $slave] {}
                $mdi::($this,core) coords icon$slave $x $y
                $mdi::($this,core) delete iconText$slave
                $mdi::($this,core) create text [expr $x + 25] [expr $y + 60] -anchor n -fill $mdi::($this,foreground) -text $mdiSlave::($slave,title) -tags iconText$slave -width 80
            }
        }
    }
            
}



# class mdiSlave --
#
#   Slave window class.  Creates a slave window in the mdi slave.
#
# Arguments (can be abreviated uniquely):
#
# Required:
#   -master - return value from [new mdi] of main window
#   -title - title to be displayed in title bar
#   -height - height of the MDI slave
#   -width - width of the MDI slave
#
# Optional:
#   -foreground and -background - the foreground and background
#       of the main window.  inherited.
#   -clicktofocus (optional) - 1 is like Windows and Mac, 0 is like some Unix.
#       inherited.
#
# Results:
#   creates new mdiSlave
class mdiSlave {
    proc mdiSlave {this args} {
        foreach {name value} $args {
            switch -glob -- $name {
                "-b*" {set mdiSlave::($this,background) $value}
                "-c*" {set mdiSlave::($this,clicktofocus) $value}
                "-f*" {set mdiSlave::($this,foreground) $value}
                "-h*" {set mdiSlave::($this,height) $value}
                "-m*" {set mdiSlave::($this,master) $value}
                "-t*" {set mdiSlave::($this,title) $value}
                "-w*" {set mdiSlave::($this,width) $value}
                "-x" {set mdiSlave::($this,x) $value}
                "-y" {set mdiSlave::($this,y) $value}
                default {bgerror "::mdiSlave class constructor : Unknown option ${name}!"}
            }
        }
        foreach name "title height width master x y" {
            if ![info exists mdiSlave::($this,$name)] {
                bgerror "::mdiSlave class constructor : No value specified for required option ${name}!"
                return 0
            }
        }
        foreach name "background clicktofocus foreground" {
            if ![info exists mdiSlave::($this,$name)] {
                set mdiSlave::($this,$name) $mdi::($mdiSlave::($this,master),$name)
            }
        }
        mdiSlave::init $this
        lappend mdi::($mdiSlave::($this,master),slaves) $this
        mdi::showSlave $mdiSlave::($this,master) $this
    }


    proc ~mdiSlave {this} {
        $mdi::($mdiSlave::($this,master),core) delete $mdiSlave::($this,window)
        destroy $mdiSlave::($this,outerFrame)
        set index [lsearch $mdi::($mdiSlave::($this,master),slaves) $this]
        set mdi::($mdiSlave::($this,master),slaves) [lreplace $mdi::($mdiSlave::($this,master),slaves) $index $index]
    }
}

#  -- mdiSlave::init
#
#   Internal procedure to initialize an mdiSlave
#
# Arguments:
#   this
# Results:
#   mdiSlave is initialized and displayed
#
proc mdiSlave::init {this} {
    image create bitmap close-button-$this -data $mdi::(images,x) -foreground white -background black
    image create bitmap max-button-$this -data $mdi::(images,max) -foreground white -background black
    image create bitmap min-button-$this  -data $mdi::(images,min) -foreground white -background black
    image create bitmap menu-button-$this -data $mdi::(images,menu) -foreground white -background black
    image create bitmap normal-button-$this -data $mdi::(images,normal)  -foreground white -background black

    set parent $mdi::($mdiSlave::($this,master),core)
    set mdiSlave::($this,outerFrame) [frame $parent.outerFrame$this -background $mdiSlave::($this,background) -width $mdiSlave::($this,width) -height $mdiSlave::($this,height) -border 0]

# Resize handles:

    set mdiSlave::($this,ridge,topleft) [frame $mdiSlave::($this,outerFrame).topleft -background $mdiSlave::($this,foreground) -height 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,top) [frame $mdiSlave::($this,outerFrame).top -background $mdiSlave::($this,foreground) -height 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,topright) [frame $mdiSlave::($this,outerFrame).topright -background $mdiSlave::($this,foreground) -height 8 -border 2 -relief raised]

    set mdiSlave::($this,ridge,bottomleft) [frame $mdiSlave::($this,outerFrame).bottomleft -background $mdiSlave::($this,foreground) -height 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,bottom) [frame $mdiSlave::($this,outerFrame).bottom -background $mdiSlave::($this,foreground) -height 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,bottomright) [frame $mdiSlave::($this,outerFrame).bottomright -background $mdiSlave::($this,foreground) -height 8 -border 2 -relief raised]

    set mdiSlave::($this,ridge,lefttop) [frame $mdiSlave::($this,outerFrame).lefttop -background $mdiSlave::($this,foreground) -width 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,left) [frame $mdiSlave::($this,outerFrame).left -background $mdiSlave::($this,foreground) -width 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,leftbottom) [frame $mdiSlave::($this,outerFrame).leftbottom -background $mdiSlave::($this,foreground) -width 8 -border 2 -relief raised]

    set mdiSlave::($this,ridge,righttop) [frame $mdiSlave::($this,outerFrame).righttop -background $mdiSlave::($this,foreground) -width 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,right) [frame $mdiSlave::($this,outerFrame).right -background $mdiSlave::($this,foreground) -width 8 -border 2 -relief raised]
    set mdiSlave::($this,ridge,rightbottom) [frame $mdiSlave::($this,outerFrame).rightbottom -background $mdiSlave::($this,foreground) -width 8 -border 2 -relief raised]

    grid $mdiSlave::($this,ridge,topleft) -row 0 -column 0 -sticky ew -columnspan 2
    grid $mdiSlave::($this,ridge,top) -row 0 -column 2 -sticky ew
    grid $mdiSlave::($this,ridge,topright) -row 0 -column 3 -columnspan 2 -sticky ew

    grid $mdiSlave::($this,ridge,bottomleft) -row 5 -column 0 -columnspan 2 -sticky ew -columnspan 2
    grid $mdiSlave::($this,ridge,bottom) -row 5 -column 2 -sticky ew
    grid $mdiSlave::($this,ridge,bottomright) -row 5 -column 3 -columnspan 2 -sticky ew

    grid $mdiSlave::($this,ridge,lefttop) -row 1 -column 0 -sticky ns
    grid $mdiSlave::($this,ridge,left) -row 2 -column 0 -sticky ns -rowspan 2
    grid $mdiSlave::($this,ridge,leftbottom) -row 4 -column 0 -sticky ns

    grid $mdiSlave::($this,ridge,righttop) -row 1 -column 4 -sticky ns
    grid $mdiSlave::($this,ridge,right) -row 2 -column 4 -sticky ns -rowspan 2
    grid $mdiSlave::($this,ridge,rightbottom) -row 4 -column 4 -sticky ns

    # Resize bindings!  Good god!

    bind $mdiSlave::($this,ridge,top) <B1-Motion> "mdiSlave::resize $this top %x %y"
    bind $mdiSlave::($this,ridge,top) <Button-1> "mdiSlave::startResize $this top %x %y"
    bind $mdiSlave::($this,ridge,top) <B1-ButtonRelease> "mdiSlave::stopResize $this top %x %y"
    bind $mdiSlave::($this,ridge,top) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor top_side"
    bind $mdiSlave::($this,ridge,top) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"

    bind $mdiSlave::($this,ridge,topright) <B1-Motion> "mdiSlave::resize $this topright %x %y"
    bind $mdiSlave::($this,ridge,topright) <Button-1> "mdiSlave::startResize $this topright %x %y"
    bind $mdiSlave::($this,ridge,topright) <B1-ButtonRelease> "mdiSlave::stopResize $this topright %x %y"
    bind $mdiSlave::($this,ridge,topright) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor top_right_corner"
    bind $mdiSlave::($this,ridge,topright) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"


    bind $mdiSlave::($this,ridge,topleft) <B1-Motion> "mdiSlave::resize $this topleft %x %y"
    bind $mdiSlave::($this,ridge,topleft) <Button-1> "mdiSlave::startResize $this topleft %x %y"
    bind $mdiSlave::($this,ridge,topleft) <B1-ButtonRelease> "mdiSlave::stopResize $this topleft %x %y"
    bind $mdiSlave::($this,ridge,topleft) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor top_left_corner"
    bind $mdiSlave::($this,ridge,topleft) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"


    bind $mdiSlave::($this,ridge,bottom) <B1-Motion> "mdiSlave::resize $this bottom %x %y"
    bind $mdiSlave::($this,ridge,bottom) <Button-1> "mdiSlave::startResize $this bottom %x %y"
    bind $mdiSlave::($this,ridge,bottom) <B1-ButtonRelease> "mdiSlave::stopResize $this bottom %x %y"
    bind $mdiSlave::($this,ridge,bottom) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor bottom_side"
    bind $mdiSlave::($this,ridge,bottom) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"


    bind $mdiSlave::($this,ridge,bottomright) <B1-Motion> "mdiSlave::resize $this bottomright %x %y"
    bind $mdiSlave::($this,ridge,bottomright) <Button-1> "mdiSlave::startResize $this bottomright %x %y"
    bind $mdiSlave::($this,ridge,bottomright) <B1-ButtonRelease> "mdiSlave::stopResize $this bottomright %x %y"
    bind $mdiSlave::($this,ridge,bottomright) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor bottom_right_corner"
    bind $mdiSlave::($this,ridge,bottomright) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"


    bind $mdiSlave::($this,ridge,bottomleft) <B1-Motion> "mdiSlave::resize $this bottomleft %x %y"
    bind $mdiSlave::($this,ridge,bottomleft) <Button-1> "mdiSlave::startResize $this bottomleft %x %y"
    bind $mdiSlave::($this,ridge,bottomleft) <B1-ButtonRelease> "mdiSlave::stopResize $this bottomleft %x %y"
    bind $mdiSlave::($this,ridge,bottomleft) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor bottom_left_corner"
    bind $mdiSlave::($this,ridge,bottomleft) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"


    bind $mdiSlave::($this,ridge,left) <B1-Motion> "mdiSlave::resize $this left %x %y"
    bind $mdiSlave::($this,ridge,left) <Button-1> "mdiSlave::startResize $this left %x %y"
    bind $mdiSlave::($this,ridge,left) <B1-ButtonRelease> "mdiSlave::stopResize $this left %x %y"
    bind $mdiSlave::($this,ridge,left) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor left_side"
    bind $mdiSlave::($this,ridge,left) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"

    bind $mdiSlave::($this,ridge,lefttop) <B1-Motion> "mdiSlave::resize $this lefttop %x %y"
    bind $mdiSlave::($this,ridge,lefttop) <Button-1> "mdiSlave::startResize $this lefttop %x %y"
    bind $mdiSlave::($this,ridge,lefttop) <B1-ButtonRelease> "mdiSlave::stopResize $this lefttop %x %y"
    bind $mdiSlave::($this,ridge,lefttop) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor top_left_corner"
    bind $mdiSlave::($this,ridge,lefttop) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"

    bind $mdiSlave::($this,ridge,leftbottom) <B1-Motion> "mdiSlave::resize $this leftbottom %x %y"
    bind $mdiSlave::($this,ridge,leftbottom) <Button-1> "mdiSlave::startResize $this leftbottom %x %y"
    bind $mdiSlave::($this,ridge,leftbottom) <B1-ButtonRelease> "mdiSlave::stopResize $this leftbottom %x %y"
    bind $mdiSlave::($this,ridge,leftbottom) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor bottom_left_corner"
    bind $mdiSlave::($this,ridge,leftbottom) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"


    bind $mdiSlave::($this,ridge,right) <B1-Motion> "mdiSlave::resize $this right %x %y"
    bind $mdiSlave::($this,ridge,right) <Button-1> "mdiSlave::startResize $this right %x %y"
    bind $mdiSlave::($this,ridge,right) <B1-ButtonRelease> "mdiSlave::stopResize $this right %x %y"
    bind $mdiSlave::($this,ridge,right) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor right_side"
    bind $mdiSlave::($this,ridge,right) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"

    bind $mdiSlave::($this,ridge,righttop) <B1-Motion> "mdiSlave::resize $this righttop %x %y"
    bind $mdiSlave::($this,ridge,righttop) <Button-1> "mdiSlave::startResize $this righttop %x %y"
    bind $mdiSlave::($this,ridge,righttop) <B1-ButtonRelease> "mdiSlave::stopResize $this righttop %x %y"
    bind $mdiSlave::($this,ridge,righttop) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor top_right_corner"
    bind $mdiSlave::($this,ridge,righttop) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"

    bind $mdiSlave::($this,ridge,rightbottom) <B1-Motion> "mdiSlave::resize $this rightbottom %x %y"
    bind $mdiSlave::($this,ridge,rightbottom) <Button-1> "mdiSlave::startResize $this rightbottom %x %y"
    bind $mdiSlave::($this,ridge,rightbottom) <B1-ButtonRelease> "mdiSlave::stopResize $this rightbottom %x %y"
    bind $mdiSlave::($this,ridge,rightbottom) <Enter> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor bottom_right_corner"
    bind $mdiSlave::($this,ridge,rightbottom) <Leave> "$mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}"

    set mdiSlave::($this,bar) [frame $mdiSlave::($this,outerFrame).bar -background $mdiSlave::($this,foreground) -height 20 -relief raised -border 2]
    menu-button-$this configure -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background)
    set leftMenu  [menubutton $mdiSlave::($this,outerFrame).leftMenu -image menu-button-$this -menu $mdiSlave::($this,outerFrame).leftMenu.wmMenu -foreground $mdiSlave::($this,background) -background $mdiSlave::($this,foreground) -padx 0 -pady 0 -border 2 -relief raised -height 20 -width 20]
    set mdiSlave::($this,menu) [menu $leftMenu.wmMenu -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background) -tearoff 0]
    $mdiSlave::($this,menu) add command -label "Minimize" -command "mdiSlave::iconize $this"
    $mdiSlave::($this,menu) add command -label "Maximize" -command "mdiSlave::maximize $this"
    $mdiSlave::($this,menu) add command -label "Close" -command "delete $this"

    min-button-$this configure -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background)
    set minimize [button $mdiSlave::($this,bar).minimize -image min-button-$this -command "mdiSlave::iconize $this" -foreground $mdiSlave::($this,background) -background $mdiSlave::($this,foreground) -padx 0 -pady 0]
    max-button-$this configure -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background)
    set maximize [button $mdiSlave::($this,bar).maximize -image max-button-$this -command {} -foreground $mdiSlave::($this,background) -background $mdiSlave::($this,foreground) -command "mdiSlave::maximize $this" -padx 0 -pady 0]
    close-button-$this configure -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background)
    set close [button $mdiSlave::($this,outerFrame).close -image close-button-$this -command "delete $this" -foreground $mdiSlave::($this,background) -background $mdiSlave::($this,foreground) -padx 0 -pady 0]
    set title [label $mdiSlave::($this,bar).title -text $mdiSlave::($this,title) -foreground $mdiSlave::($this,background) -background $mdiSlave::($this,foreground) -pady 0]

    grid $leftMenu -row 1 -column 1 -sticky nwes

    grid $title -row 0 -column 1 -sticky nsew
    grid $minimize -row 0 -column 2 -sticky ne
    grid $maximize -row 0 -column 3 -sticky ne

    grid $close -row 1 -column 3 -sticky nse

    grid rowconfigure $mdiSlave::($this,bar) 0 -weight 0
    grid columnconfigure $mdiSlave::($this,bar) 1 -weight 1

    bind $title <B1-Motion> "mdiSlave::move $this %x %y; break;"
    bind $title <Button-1> "mdiSlave::startMove $this %x %y; break;"
    bind $title <B1-ButtonRelease> "mdiSlave::stopMove $this; break;"
    bind $title <Button-3> "mdiSlave::lower $this; break;"

    set mdiSlave::($this,frame) [frame $mdiSlave::($this,outerFrame).frame -background $mdiSlave::($this,background) -width $mdiSlave::($this,width) -height $mdiSlave::($this,height)]

    grid $mdiSlave::($this,bar) -row 1 -column 2 -sticky new
    grid $mdiSlave::($this,frame) -row 3 -column 2 -sticky nsew -rowspan 2
   
    grid rowconfigure $mdiSlave::($this,outerFrame) 0 -weight 0
    grid rowconfigure $mdiSlave::($this,outerFrame) 1 -weight 0
    grid rowconfigure $mdiSlave::($this,outerFrame) 2 -weight 0
    grid rowconfigure $mdiSlave::($this,outerFrame) 3 -weight 1
    grid rowconfigure $mdiSlave::($this,outerFrame) 4 -weight 0 -minsize 20
    grid rowconfigure $mdiSlave::($this,outerFrame) 5 -weight 0

    grid columnconfigure $mdiSlave::($this,outerFrame) 0 -weight 0
    grid columnconfigure $mdiSlave::($this,outerFrame) 1 -weight 0
    grid columnconfigure $mdiSlave::($this,outerFrame) 2 -weight 1
    grid columnconfigure $mdiSlave::($this,outerFrame) 3 -weight 0
    grid columnconfigure $mdiSlave::($this,outerFrame) 4 -weight 0

    set mdiSlave::($this,maximized) 0
    set mdiSlave::($this,iconized) 0
    update
}

#  -- mdiSlave::iconize
#
#   Iconizes an mdiSlave object.
#
# Arguments:
#   this
# Results:
#   mdiSlave is iconized
#
proc mdiSlave::iconize {this} {
    foreach {x y} [mdiSlave::findIconSpot $this] {}

    image create bitmap icon-$this -data $mdi::(images,icon) -foreground $mdiSlave::($this,background) -background $mdiSlave::($this,foreground) 

    $mdi::($mdiSlave::($this,master),core) create image $x $y -image icon-$this -anchor nw -tags icon$this
    $mdi::($mdiSlave::($this,master),core) create text [expr $x + 25] [expr $y + 60] -anchor n -fill $mdi::($mdiSlave::($this,master),foreground) -text $mdiSlave::($this,title) -tags iconText$this -width 80
    $mdi::($mdiSlave::($this,master),core) delete $mdiSlave::($this,window)

    $mdi::($mdiSlave::($this,master),core) bind icon$this <B1-Motion> "mdiSlave::iconMove $this %x %y; break;"
    $mdi::($mdiSlave::($this,master),core) bind icon$this <Button-1> "mdiSlave::startIconMove $this %x %y; break;"
    $mdi::($mdiSlave::($this,master),core) bind icon$this <B1-ButtonRelease> "mdiSlave::stopIconMove $this; break;"
    set mdiSlave::($this,iconized) 1
}

# mdiSlave::startIconMove --
#
#   internal proc - helps icon move
#
# Arguments:
#   this x y
# Results:
#   icon moves!
#
proc mdiSlave::startIconMove {this x y} {
    set c $mdi::($mdiSlave::($this,master),core)
    foreach {windowx windowy windowx2 windowy2} [$c bbox icon$this] {}
    $c create rectangle $windowx $windowy $windowx2 $windowy2 -outline $mdi::($mdiSlave::($this,master),foreground) -tags moveRect$this
    set mdiSlave::($this,lastIconx) $windowx
    set mdiSlave::($this,lastIcony) $windowy
    set mdiSlave::($this,moveSizeIconX) [expr $windowx2 - $windowx]
    set mdiSlave::($this,moveSizeIconY) [expr $windowy2 - $windowy]
    set mdiSlave::($this,offsetIconX) [expr $x - $windowx]
    set mdiSlave::($this,offsetIconY) [expr $y - $windowy]
}

# mdiSlave::iconMove --
#
#   internal proc - helps icon move
#
# Arguments:
#   this x y
# Results:
#   icon moves!
#
proc mdiSlave::iconMove {this x y} {
    set c $mdi::($mdiSlave::($this,master),core)
    set x [expr $x - $mdiSlave::($this,offsetIconX)]
    set y [expr $y - $mdiSlave::($this,offsetIconY)]
    $c coords moveRect$this $x $y [expr $x + $mdiSlave::($this,moveSizeIconX)] [expr $y + $mdiSlave::($this,moveSizeIconY)]
}

# mdiSlave::startIconMove --
#
#   internal proc - helps icon move
#
# Arguments:
#   this x y
# Results:
#   icon moves!
#
proc mdiSlave::stopIconMove {this} {
    set c $mdi::($mdiSlave::($this,master),core)
    foreach {newx newy trash1 trash2} [$c coords moveRect$this] {}
    if {($mdiSlave::($this,lastIconx) == $newx) && ($mdiSlave::($this,lastIcony) == $newy)} {
        mdi::showSlave $mdiSlave::($this,master) $this
        mdiSlave::raise $this
        $c delete icon$this
        $c delete iconText$this
        $c delete moveRect$this
        mdiSlave::freeIconSpot $this
        set mdiSlave::($this,iconized) 0
        return
    }
    $c delete moveRect$this
    $c delete iconText$this
    $c coords icon$this $newx $newy
    $c create text [expr $newx + 25] [expr $newy + 60] -anchor n -fill $mdi::($mdiSlave::($this,master),foreground) -text $mdiSlave::($this,title) -tags iconText$this -width 80
}

# mdiSlave::freeIconSpot --
#
#   Notifies the icon spot allocator that "this" slave is done with its spot.
#
# Arguments:
#   this
# Results:
#   Nothing
#
proc mdiSlave::freeIconSpot {this} {
    set master $mdiSlave::($this,master)
    lappend mdi::($master,iconSpots) $mdi::($master,$this,iconLoc)
    unset mdi::($master,$this,iconLoc)
}


# mdiSlave::findIconSpot --
#
#   Finds a place to put an icon.
#
# Arguments:
#   this
# Results:
#   [list x y]  : place to put the icon
#
proc mdiSlave::findIconSpot {this} {
    set master $mdiSlave::($this,master)
    set c $mdi::($master,core)

    if ![info exists mdi::($master,iconSpots)] {
        update
        for {set ySize [winfo height $c]; set y 0} {$y <= [expr $ySize - 100]} {incr y 100} {
            for {set xSize [winfo width $c]; set x [expr $xSize - 90] } {$x >= 10} {incr x -100} {
                lappend mdi::($master,iconSpots) [list $x $y]
            }
        }
        if ![info exists mdi::($master,iconSpots)] {
            set mdi::($master,iconSpots) [list [list 0 0]]
        }
    }

    if {[llength $mdi::($master,iconSpots)] == 1} {
        set mdi::($master,$this,iconLoc) [list [list 0 0]]
        return [list 0 0]
    } else {
        set mdi::($master,$this,iconLoc) [lindex $mdi::($master,iconSpots) end]
        set mdi::($master,iconSpots) [lrange $mdi::($master,iconSpots) 0 [expr [llength $mdi::($master,iconSpots)] - 2]]
        return $mdi::($master,$this,iconLoc)
    }

}

# mdiSlave::startResize --
#
#   Internal procedure - handles user resize.
#
# Arguments:
#   this...
# Results:
#   handles resizing.
#
proc mdiSlave::startResize {this which x y} {
    $mdi::($mdiSlave::($this,master),toplevel) configure -cursor sizing
    mdiSlave::raise $this
    set c $mdi::($mdiSlave::($this,master),core)
    foreach {windowx1 windowy1 windowx2 windowy2} [$c bbox $mdiSlave::($this,window)] {}
    $c create rectangle $windowx1 $windowy1 $windowx2 $windowy2 -outline $mdi::($mdiSlave::($this,master),foreground) -tags resizeRect$this
    set mdiSlave::($this,lastx1) $windowx1
    set mdiSlave::($this,lasty1) $windowy1
    set mdiSlave::($this,lastx2) $windowx2
    set mdiSlave::($this,lasty2) $windowy2
    set mdiSlave::($this,offsetX1) [expr $x - $windowx1]
    set mdiSlave::($this,offsetY1) [expr $y - $windowy1]
    set mdiSlave::($this,offsetX2) [expr $x - $windowx2]
    set mdiSlave::($this,offsetY2) [expr $y - $windowy2]
}
# mdiSlave::resize --
#
#   Internal procedure - handles user resize.
#
# Arguments:
#   this...
# Results:
#   handles resizing.
#
proc mdiSlave::resize {this which x y} {
    $mdi::($mdiSlave::($this,master),toplevel) configure -cursor sizing
    set c $mdi::($mdiSlave::($this,master),core)
    set x1 $mdiSlave::($this,lastx1)
    set x2 $mdiSlave::($this,lastx2)
    set y1 $mdiSlave::($this,lasty1)
    set y2 $mdiSlave::($this,lasty2)
    if {[string match "topleft" $which] || [string match "lefttop" $which]} {
        set x1 [expr $x - $mdiSlave::($this,offsetX1)]
        set y1 [expr $y - $mdiSlave::($this,offsetY1)]
    } elseif {[string match "topright" $which] || [string match "righttop" $which]} {
        set x2 [expr $x - $mdiSlave::($this,offsetX2)]
        set y1 [expr $y - $mdiSlave::($this,offsetY1)]
    } elseif  {[string match "rightbottom" $which] || [string match "bottomright" $which]} {
        set x2 [expr $x - $mdiSlave::($this,offsetX2)]
        set y2 [expr $y - $mdiSlave::($this,offsetY2)]
    } elseif  {[string match "leftbottom" $which] || [string match "bottomleft" $which] } {
        set x1 [expr $x - $mdiSlave::($this,offsetX1)]
        set y2 [expr $y - $mdiSlave::($this,offsetY1)]
    } elseif {[string match "top" $which]} {
        set y1 [expr $y - $mdiSlave::($this,offsetY1)]
    } elseif {[string match "bottom" $which]} {
        set y2 [expr $y - $mdiSlave::($this,offsetY2)]
    } elseif {[string match "left" $which]} {
        set x1 [expr $x - $mdiSlave::($this,offsetX1)]
    } elseif {[string match "right" $which]} {
        set x2 [expr $x - $mdiSlave::($this,offsetX2)]
    }
    $c coords resizeRect$this $x1 $y1 $x2 $y2
}

# mdiSlave::stopResize --
#
#   Internal procedure - handles user resize.
#
# Arguments:
#   this...
# Results:
#   handles resizing.
#
proc mdiSlave::stopResize {this which x y} {
    $mdi::($mdiSlave::($this,master),toplevel) configure -cursor {}
    set c $mdi::($mdiSlave::($this,master),core)
    foreach {x1 y1 x2 y2} [$c coords resizeRect$this] {}
    $c delete resizeRect$this
    mdiSlave::changecoords $this $x1 $y1
    mdiSlave::changesize $this [expr $x2 - $x1] [expr $y2 - $y1]
    set mdiSlave::($this,height) [expr $y2 - $y1]
    set mdiSlave::($this,width) [expr $x2 - $x1]
    set mdiSlave::($this,x) $x1
    set mdiSlave::($this,y) $y1
}

    
# mdiSlave::startMove --
#
#   Internal procedure, helps user move mdiSlave
#
# Arguments:
#   this...
# Results:
#   user moves window!
#
proc mdiSlave::startMove {this x y} {
    if $mdiSlave::($this,clicktofocus) {
        mdiSlave::raise $this
    }
    set c $mdi::($mdiSlave::($this,master),core)
    foreach {windowx windowy windowx2 windowy2} [$c bbox $mdiSlave::($this,window)] {}
    $c create rectangle $windowx $windowy $windowx2 $windowy2 -outline $mdi::($mdiSlave::($this,master),foreground) -tags moveRect$this
    set mdiSlave::($this,lastx) $windowx
    set mdiSlave::($this,lasty) $windowy
    set mdiSlave::($this,moveSizeX) [expr $windowx2 - $windowx]
    set mdiSlave::($this,moveSizeY) [expr $windowy2 - $windowy]
    set mdiSlave::($this,offsetX) [expr $x - $windowx]
    set mdiSlave::($this,offsetY) [expr $y - $windowy]
}

# mdiSlave::move --
#
#   Internal procedure, helps user move mdiSlave
#
# Arguments:
#   this...
# Results:
#   user moves window!
#
proc mdiSlave::move {this x y} {
    set c $mdi::($mdiSlave::($this,master),core)
    set x [expr $x - $mdiSlave::($this,offsetX)]
    set y [expr $y - $mdiSlave::($this,offsetY)]
    $c coords moveRect$this $x $y [expr $x + $mdiSlave::($this,moveSizeX)] [expr $y + $mdiSlave::($this,moveSizeY)]
}

# mdiSlave::stopMove --
#
#   Internal procedure, helps user move mdiSlave
#
# Arguments:
#   this...
# Results:
#   user moves window!
#
proc mdiSlave::stopMove {this} {
    set c $mdi::($mdiSlave::($this,master),core)
    foreach {newx newy trash1 trash2} [$c coords moveRect$this] {}
    if {($mdiSlave::($this,lastx) == $newx) && ($mdiSlave::($this,lasty) == $newy)} {
        if !$mdiSlave::($this,clicktofocus) {
            mdiSlave::raise $this
        }
    }
    $c delete moveRect$this
    $c coords $mdiSlave::($this,window) $newx $newy
    set mdiSlave::($this,x) $newx
    set mdiSlave::($this,y) $newy
}

#  -- mdiSlave::raise
#
#   raises an mdiSlave window
#
# Arguments:
#   this
# Results:
#   window is above all others
#
proc mdiSlave::raise {this} {
    ::raise $mdiSlave::($this,outerFrame)
    focus $mdiSlave::($this,frame)
}

#  -- mdiSlave::lower
#
#   lower an mdiSlave window
#
# Arguments:
#   this
# Results:
#   window is below all others
#
 proc mdiSlave::lower {this} {
    ::lower $mdiSlave::($this,outerFrame)
}

#  -- mdiSlave::maximize
#
#   makes mdiSlave fill screen
#
# Arguments:
#   this
# Results:
#   mdiSlave fills screen
#
proc mdiSlave::maximize {this} {
    set mdiSlave::($this,oldHeight) [winfo height $mdiSlave::($this,outerFrame)]
    set mdiSlave::($this,oldWidth) [winfo width $mdiSlave::($this,outerFrame)]
    $mdi::($mdiSlave::($this,master),core) coords $mdiSlave::($this,window) 0 0
    $mdi::($mdiSlave::($this,master),core) itemconfigure $mdiSlave::($this,window) -width [winfo width $mdi::($mdiSlave::($this,master),core)] -height [winfo height $mdi::($mdiSlave::($this,master),core)]
    $mdiSlave::($this,outerFrame) configure -width [winfo width $mdi::($mdiSlave::($this,master),core)] -height [winfo height $mdi::($mdiSlave::($this,master),core)]
    normal-button-$this configure -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background)
    $mdiSlave::($this,bar).maximize configure -image normal-button-$this -command "mdiSlave::unmaximize $this"

    set mdiSlave::($this,oldX) $mdiSlave::($this,x)
    set mdiSlave::($this,oldY) $mdiSlave::($this,y)

    set mdiSlave::($this,height) [winfo height $mdi::($mdiSlave::($this,master),core)]
    set mdiSlave::($this,width) [winfo width $mdi::($mdiSlave::($this,master),core)]
    set mdiSlave::($this,x) 0
    set mdiSlave::($this,y) 0

    set mdiSlave::($this,maximized) 1

    mdiSlave::raise $this
}

# mdiSlave::unmaximize --
#
#   unMaximizes (see above) mdiSlave window
#
# Arguments:
#   this
# Results:
#   mdiSlave returns to previous size
#
proc mdiSlave::unmaximize {this} {
    $mdi::($mdiSlave::($this,master),core) coords $mdiSlave::($this,window) $mdiSlave::($this,oldX) $mdiSlave::($this,oldY)
    $mdi::($mdiSlave::($this,master),core) itemconfigure $mdiSlave::($this,window) -width $mdiSlave::($this,oldWidth) -height $mdiSlave::($this,oldHeight)
    $mdiSlave::($this,outerFrame) configure -width $mdiSlave::($this,oldWidth) -height $mdiSlave::($this,oldHeight)
    max-button-$this configure -background $mdiSlave::($this,foreground) -foreground $mdiSlave::($this,background)
    $mdiSlave::($this,bar).maximize configure -image max-button-$this -command "mdiSlave::maximize $this"

    set mdiSlave::($this,height) $mdiSlave::($this,oldHeight)
    set mdiSlave::($this,width) $mdiSlave::($this,oldWidth)
    set mdiSlave::($this,x) $mdiSlave::($this,oldX)
    set mdiSlave::($this,y) $mdiSlave::($this,oldY)

    set mdiSlave::($this,maximized) 0

    mdiSlave::raise $this
}

# mdiSlave::changecoords --
#
#   Internal proc - changs coords of mdiSlave
#
# Arguments:
#   this x y
# Results:
#   mdiSlave moves!
#
proc mdiSlave::changecoords {this x y} {
    $mdi::($mdiSlave::($this,master),core) coords $mdiSlave::($this,window) $x $y
}

# mdiSlave::changesize --
#
#   Internal proc - changs size of mdiSlave
#
# Arguments:
#   this x y
# Results:
#   mdiSlave resizes!
#
proc mdiSlave::changesize {this sizex sizey} {
    $mdi::($mdiSlave::($this,master),core) itemconfigure $mdiSlave::($this,window) -width $sizex -height $sizey
    $mdiSlave::($this,outerFrame) configure -width $sizex -height $sizey
}

# mdi::globalInit --
#
#   Internal procedure to initialize global data.  Currently just bitmaps.
#
# Arguments:
#   None.
# Results:
#   ::mdi class globals are initialized
#
proc mdi::globalInit {} {
    set mdi::(globalInitDone) 1
    set mdi::(images,x) {
#define x_width 16
#define x_height 16
static unsigned char x_bits[] = {
   0x07, 0xe0, 0x0f, 0xf0, 0x1e, 0x78, 0x3c, 0x3c, 0x78, 0x1e, 0xf0, 0x0f,
   0xe0, 0x07, 0xc0, 0x03, 0xe0, 0x07, 0xf0, 0x0f, 0x78, 0x1e, 0x3c, 0x3c,
   0x1e, 0x78, 0x0f, 0xf0, 0x07, 0xe0, 0x03, 0xc0};
}
   set mdi::(images,max) {
#define max_width 16
#define max_height 16
static unsigned char max_bits[] = {
   0x00, 0x00, 0xfe, 0x7f, 0xfe, 0x7f, 0x06, 0x70, 0x06, 0x70, 0x06, 0x70,
   0x06, 0x70, 0x06, 0x70, 0x06, 0x70, 0x06, 0x70, 0x06, 0x70, 0x06, 0x70,
   0xfe, 0x7f, 0xfe, 0x7f, 0xfe, 0x7f, 0x00, 0x00};
    }
    set mdi::(images,min) {
#define min.xbm_width 16
#define min.xbm_height 16
static unsigned char min.xbm_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x07, 0x20, 0x06,
   0x20, 0x06, 0x20, 0x06, 0xe0, 0x07, 0xe0, 0x07, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    }
    set mdi::(images,menu) {
#define menu.xbm_width 16
#define menu.xbm_height 16
static unsigned char menu.xbm_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0xfe, 0x3f, 0x02, 0x60, 0xfe, 0x7f, 0xfc, 0x7f, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00};
    }
    set mdi::(images,normal) {
#define normal_width 16
#define normal_height 16
static unsigned char normal_bits[] = {
   0x00, 0x00, 0xf0, 0x7f, 0x10, 0x60, 0x10, 0x60, 0x10, 0x60, 0x10, 0x60,
   0x1e, 0x60, 0xf2, 0x7f, 0xf2, 0x7f, 0x02, 0x0c, 0x02, 0x0c, 0x02, 0x0c,
   0x02, 0x0c, 0xfe, 0x0f, 0xfe, 0x0f, 0x00, 0x00};
    }
    set mdi::(images,icon) {
#define icon_width 50
#define icon_height 50
static unsigned char icon_bits[] = {
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xfe, 0xff, 0xff,
   0xff, 0xff, 0xff, 0x01, 0x42, 0x00, 0x00, 0x00, 0x42, 0x98, 0x01, 0x5a,
   0xf0, 0xff, 0x7f, 0x42, 0x2b, 0x01, 0x42, 0xf0, 0xff, 0x7f, 0x52, 0x4b,
   0x01, 0x42, 0x00, 0x00, 0x00, 0x42, 0x98, 0x01, 0xfe, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x01, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x01, 0xfe, 0xff, 0xff, 0xff, 0xff, 0xff, 0x01,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00};
    }
}
